/*
Copyright (c) 2015 Red Hat, Inc.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package org.ovirt.engine.sdk4.services;

import java.io.IOException;
import java.math.BigInteger;
import java.util.List;
import org.ovirt.engine.sdk4.Request;
import org.ovirt.engine.sdk4.Response;
import org.ovirt.engine.sdk4.Service;
import org.ovirt.engine.sdk4.builders.TemplateBuilder;
import org.ovirt.engine.sdk4.types.Template;

/**
 * This service manages the virtual machine templates available in the system.
 */
public interface TemplatesService extends Service {
    /**
     * Creates a new template.
     * 
     * This requires the `name` and `vm` elements. To identify the virtual machine use the `vm.id` or `vm.name`
     * attributes. For example, to create a template from virtual machine with identifier `123` send a request
     * like this:
     * 
     * [source]
     * ----
     * POST /ovirt-engine/api/templates
     * ----
     * 
     * With a request body like this:
     * 
     * [source,xml]
     * ----
     * <template>
     *   <name>mytemplate</name>
     *   <vm id="123"/>
     * </template>
     * ----
     * 
     * The disks of the template can be customized, making some of their characteristics different to the disks of the
     * original virtual machine. To do so use the `vm.disk_attachments` attribute, specifying the identifier of the disk
     * of the original virtual machine and the characteristics that you want to change. For example, if the original
     * virtual machine has a disk with identifier `456`, and, for that disk, you want to change the format to
     * <<types/disk_format, _Copy On Write_>> and make the disk <<types/disk, sparse>>, send a request body
     * like this:
     * 
     * [source,xml]
     * ----
     * <template>
     *   <name>mytemplate</name>
     *   <vm id="123">
     *     <disk_attachments>
     *       <disk_attachment>
     *         <disk id="456">
     *           <format>cow</format>
     *           <sparse>true</sparse>
     *         </disk>
     *       </disk_attachment>
     *     </disk_attachments>
     *   </vm>
     * </template>
     * ----
     * 
     * The template can be created as a sub version of an existing template.This requires the `name` and `vm` attributes
     * for the new template, and the `base_template` and `version_name` attributes for the new template version. The
     * `base_template` and `version_name` attributes must be specified within a `version` section enclosed in the
     * `template` section. Identify the virtual machine with the `id` or `name` attributes.
     * 
     * [source,xml]
     * ----
     * <template>
     *   <name>mytemplate</name>
     *   <vm id="123"/>
     *   <version>
     *     <base_template id="456"/>
     *     <version_name>mytemplate_001</version_name>
     *   </version>
     * </template>
     * ----
     */
    public interface AddRequest extends Request<AddRequest, AddResponse> {
        /**
         * Specifies if the permissions of the virtual machine should be copied to the template.
         * 
         * If this optional parameter is provided, and its values is `true` then the permissions of the virtual machine
         * (only the direct ones, not the inherited ones) will be copied to the created template. For example, to create
         * a template from the `myvm` virtual machine copying its permissions, send a request like this:
         * 
         * [source]
         * ----
         * POST /ovirt-engine/api/templates?clone_permissions=true
         * ----
         * 
         * With a request body like this:
         * 
         * [source,xml]
         * ----
         * <template>
         *   <name>mytemplate<name>
         *   <vm>
         *     <name>myvm<name>
         *   </vm>
         * </template>
         * ----
         */
        AddRequest clonePermissions(Boolean clonePermissions);
        /**
         * Seal the template.
         * 
         * If this optional parameter is provided and its value is `true`,
         * then the template is sealed after creation.
         * 
         * Sealing erases all host-specific configuration from the filesystem:
         * SSH keys, UDEV rules, MAC addresses, system ID, hostname etc.,
         * thus making easy to use the template to create multiple virtual
         * machines without manual intervention.
         * 
         * Currently sealing is supported only for Linux OS.
         */
        AddRequest seal(Boolean seal);
        AddRequest template(Template template);
        /**
         * The information about the template or template version.
         */
        AddRequest template(TemplateBuilder template);
    }
    
    /**
     * Creates a new template.
     * 
     * This requires the `name` and `vm` elements. To identify the virtual machine use the `vm.id` or `vm.name`
     * attributes. For example, to create a template from virtual machine with identifier `123` send a request
     * like this:
     * 
     * [source]
     * ----
     * POST /ovirt-engine/api/templates
     * ----
     * 
     * With a request body like this:
     * 
     * [source,xml]
     * ----
     * <template>
     *   <name>mytemplate</name>
     *   <vm id="123"/>
     * </template>
     * ----
     * 
     * The disks of the template can be customized, making some of their characteristics different to the disks of the
     * original virtual machine. To do so use the `vm.disk_attachments` attribute, specifying the identifier of the disk
     * of the original virtual machine and the characteristics that you want to change. For example, if the original
     * virtual machine has a disk with identifier `456`, and, for that disk, you want to change the format to
     * <<types/disk_format, _Copy On Write_>> and make the disk <<types/disk, sparse>>, send a request body
     * like this:
     * 
     * [source,xml]
     * ----
     * <template>
     *   <name>mytemplate</name>
     *   <vm id="123">
     *     <disk_attachments>
     *       <disk_attachment>
     *         <disk id="456">
     *           <format>cow</format>
     *           <sparse>true</sparse>
     *         </disk>
     *       </disk_attachment>
     *     </disk_attachments>
     *   </vm>
     * </template>
     * ----
     * 
     * The template can be created as a sub version of an existing template.This requires the `name` and `vm` attributes
     * for the new template, and the `base_template` and `version_name` attributes for the new template version. The
     * `base_template` and `version_name` attributes must be specified within a `version` section enclosed in the
     * `template` section. Identify the virtual machine with the `id` or `name` attributes.
     * 
     * [source,xml]
     * ----
     * <template>
     *   <name>mytemplate</name>
     *   <vm id="123"/>
     *   <version>
     *     <base_template id="456"/>
     *     <version_name>mytemplate_001</version_name>
     *   </version>
     * </template>
     * ----
     */
    public interface AddResponse extends Response {
        /**
         * The information about the template or template version.
         */
        Template template();
    }
    
    /**
     * Creates a new template.
     * 
     * This requires the `name` and `vm` elements. To identify the virtual machine use the `vm.id` or `vm.name`
     * attributes. For example, to create a template from virtual machine with identifier `123` send a request
     * like this:
     * 
     * [source]
     * ----
     * POST /ovirt-engine/api/templates
     * ----
     * 
     * With a request body like this:
     * 
     * [source,xml]
     * ----
     * <template>
     *   <name>mytemplate</name>
     *   <vm id="123"/>
     * </template>
     * ----
     * 
     * The disks of the template can be customized, making some of their characteristics different to the disks of the
     * original virtual machine. To do so use the `vm.disk_attachments` attribute, specifying the identifier of the disk
     * of the original virtual machine and the characteristics that you want to change. For example, if the original
     * virtual machine has a disk with identifier `456`, and, for that disk, you want to change the format to
     * <<types/disk_format, _Copy On Write_>> and make the disk <<types/disk, sparse>>, send a request body
     * like this:
     * 
     * [source,xml]
     * ----
     * <template>
     *   <name>mytemplate</name>
     *   <vm id="123">
     *     <disk_attachments>
     *       <disk_attachment>
     *         <disk id="456">
     *           <format>cow</format>
     *           <sparse>true</sparse>
     *         </disk>
     *       </disk_attachment>
     *     </disk_attachments>
     *   </vm>
     * </template>
     * ----
     * 
     * The template can be created as a sub version of an existing template.This requires the `name` and `vm` attributes
     * for the new template, and the `base_template` and `version_name` attributes for the new template version. The
     * `base_template` and `version_name` attributes must be specified within a `version` section enclosed in the
     * `template` section. Identify the virtual machine with the `id` or `name` attributes.
     * 
     * [source,xml]
     * ----
     * <template>
     *   <name>mytemplate</name>
     *   <vm id="123"/>
     *   <version>
     *     <base_template id="456"/>
     *     <version_name>mytemplate_001</version_name>
     *   </version>
     * </template>
     * ----
     */
    AddRequest add();
    
    /**
     * Returns the list of virtual machine templates.
     * 
     * For example:
     * 
     * [source]
     * ----
     * GET /ovirt-engine/api/templates
     * ----
     * 
     * Will return the list of virtual machines and virtual machine templates.
     * 
     * The order of the returned list of templates isn't guaranteed.
     */
    public interface ListRequest extends Request<ListRequest, ListResponse> {
        /**
         * Indicates if the search performed using the `search` parameter should be performed taking case into
         * account. The default value is `true`, which means that case is taken into account. If you want to search
         * ignoring case set it to `false`.
         */
        ListRequest caseSensitive(Boolean caseSensitive);
        /**
         * Indicates if the results should be filtered according to the permissions of the user.
         */
        ListRequest filter(Boolean filter);
        /**
         * Sets the maximum number of templates to return. If not specified all the templates are returned.
         */
        ListRequest max(Integer max);
        /**
         * Sets the maximum number of templates to return. If not specified all the templates are returned.
         */
        ListRequest max(Long max);
        /**
         * Sets the maximum number of templates to return. If not specified all the templates are returned.
         */
        ListRequest max(BigInteger max);
        /**
         * A query string used to restrict the returned templates.
         */
        ListRequest search(String search);
    }
    
    /**
     * Returns the list of virtual machine templates.
     * 
     * For example:
     * 
     * [source]
     * ----
     * GET /ovirt-engine/api/templates
     * ----
     * 
     * Will return the list of virtual machines and virtual machine templates.
     * 
     * The order of the returned list of templates isn't guaranteed.
     */
    public interface ListResponse extends Response {
        /**
         * The list of virtual machine templates.
         */
        List<Template> templates();
    }
    
    /**
     * Returns the list of virtual machine templates.
     * 
     * For example:
     * 
     * [source]
     * ----
     * GET /ovirt-engine/api/templates
     * ----
     * 
     * Will return the list of virtual machines and virtual machine templates.
     * 
     * The order of the returned list of templates isn't guaranteed.
     */
    ListRequest list();
    
    /**
     * Returns a reference to the service that manages a specific virtual machine template.
     */
    TemplateService templateService(String id);
    /**
     * Service locator method, returns individual service on which the URI is dispatched.
     */
    Service service(String path);
}

